#! /usr/bin/perl -w
## dep.pl - The Dependency Analyst
## Copyright (C) 1999, 2000 The MadHouse Project
##
## This file is part of dep.pl.
##
## dep.pl is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## dep.pl is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
## $Id: dep.pl.in,v 1.36 2000/12/21 15:59:34 algernon Exp $

require 5.004;

use DepPl;
use Getopt::Long;
use strict;
use sigtrap 'handler' => \&hup_trap, 'HUP';

##########################################
# SETUP GLOBALS                          #
##########################################
my $userconfdir=$ENV{'HOME'}."/.dep.pl";
my %deps=();
my %libs=();
my %misslibs=();
my %checked=();
my %stats=();
my %exclude=();
my $VERSION=do {my @r=(q$Revision: 1.36 $=~/\d+/g); sprintf "%d."."%02d" x $#r, @r};
my $version="dep.pl $VERSION (dep.pl 1.36.0-5)\n";
my $copyright=$version."\n".
	"Copyright (C) 1999, 2000 The MadHouse Project\n".
	"This is free software; see the source for copying conditions.  There is NO\n".
	"warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n";
my $banner=$version."-"x(length($version)-1)."\n\n";
my $help="Usage: dep.pl [OPTIONS] files...\n\n".
              "OPTIONS:\n".
	      "\t--bare,               -b : quite like --libs.\n".
	      "\t--check-area=[path],  -c : skip files outside [path].\n".
	      "\t--chroot=[path],      -C : simulate a chroot into [path].\n".
	      "\t--err,                -e : output status to STDERR, result to STDOUT.\n".
	      "\t--exclude=[files],    -x : colon separated list of files to exclude.\n".
	      "\t--force=[ln],         -f : force use of the [ln] language.\n".
	      "\t--help,               -h : this help screen.\n".
	      "\t--libs,               -l : report only library filenames.\n".
	      "\t--license,            -L : displays license, then exits.\n".
	      "\t--nohup,              -n : Ignore HUP signal.\n".
	      "\t--recursive,          -R : operate in recursive mode.\n".
	      "\t--report,             -r : generate full report (default action).\n".
	      "\t--stat=[level],       -s : display some [level] detailed statistics.\n".
	      "\t--status=[boolean],   -S : turns status messages on/off.\n".
	      "\t--quiet,              -q : operate quietly.\n".
	      "\t--version,            -v : print version information, then exit.\n\n";

$DepPl::messages{"help"}=$help;

$stats{"start_time"}=time;

# These are used later.
my $file;
my $result;
my $path;
my $lib;
my $newfile;
my $newfn;

my $cwd=$ENV{"PWD"};

##########################################
# LOAD CONFIGURATION                     #
##########################################

$DepPl::args{"force"}="en";
DepPl::loadconfig($DepPl::confdir);
DepPl::loadsysconf();

##########################################
# GET OPTIONS                            #
##########################################
Getopt::Long::config ("no_ignore_case");
Getopt::Long::config ("bundling");
Getopt::Long::config ("no_auto_abbrev");

# Remove the comment from the next line, if you want to...
#Getopt::Long::config ("pass_through");

my $optval = GetOptions (
	"help|h", \$DepPl::args{"help"},
	"version|v", \$DepPl::args{"version"},
	"license|L", \$DepPl::args{"license"},
	"quiet|q", \$DepPl::args{"quiet"},
	"report|r", \$DepPl::args{"report"},
	"recursive|R", \$DepPl::args{"recursive"},
	"libs|l", \$DepPl::args{"libs"},
	"bare|b", \$DepPl::args{"bare"},
	"err|e", \$DepPl::args{"err"},
	"force|f=s", \$DepPl::args{"force"},
	"stat|s:i", \$DepPl::args{"stat"},
	"status|S=s", \$DepPl::args{"status"},
	"check-area|c=s", \$DepPl::args{"check-area"},
	"exclude|x=s", \$DepPl::args{"exclude"},
	"chroot|C=s", \$DepPl::args{"chroot"},
	"nohup|n", \$DepPl::args{"nohup"},
);

# These are here for compatibility reasons.
if (defined $DepPl::args{"force"}) {
	$DepPl::args{"force"}=~s/^=//g;
}
if (defined $DepPl::args{"check-area"}) {
	$DepPl::args{"check-area"}=~s/^=//g;
}
if (defined $DepPl::args{"exclude"}) {
	$DepPl::args{"exclude"}=~s/^=//g;
}
if (defined $DepPl::args{"chroot"}) {
	$DepPl::args{"chroot"}=~s/^=//g;
}

# bare =~ libs + err
if ($DepPl::args{"bare"}) {
	$DepPl::args{"err"}=1;
}

# if --err, select STDERR...
if ($DepPl::args{"err"}) {
	select (STDERR);
	$DepPl::args{"libs"}=1;
}

# if report is specified _explictly_, turn off everything else
if ($DepPl::args{"report"}) {
	undef ($DepPl::args{"libs"});
	undef ($DepPl::args{"quiet"});
}

# if quiet or bare, turn off statistics
if ( $DepPl::args{"quiet"} || $DepPl::args{"bare"}) {
	undef ($DepPl::args{"stat"});
}

# report is the default mode
if (!$DepPl::args{"report"} && !$DepPl::args{"err"} &&
    !$DepPl::args{"libs"} && !$DepPl::args{"quiet"}) {
	$DepPl::args{"report"}=1;
}

# status parsing (should be a boolean)
if (defined($DepPl::args{"status"}) && 
    $DepPl::args{"status"}=~/^(off|false|0|no)$/i) {
	$DepPl::args{"status"}=0;
} else {
	$DepPl::args{"status"}=1;
}

# --exclude parsing
if (defined($DepPl::args{"exclude"})) {
	$DepPl::args{"exclude"}=~s/^=//g;
	my @files=split(/:/,$DepPl::args{"exclude"});
	my $file;
	foreach $file (@files) {
		$exclude{$file}=1;
	}
}

# expand --chroot and --checking-area args...
if (defined($DepPl::args{"chroot"}) && $DepPl::args{"chroot"}!~/^\//) {
	$DepPl::args{"chroot"}=DepPl::resolve_path($cwd."/".$DepPl::args{"chroot"});
}

if (defined($DepPl::args{"check-area"}) && 
    $DepPl::args{"check-area"}!~/^\//) {
	$DepPl::args{"check-area"}=DepPl::resolve_path($cwd."/".$DepPl::args{"check-area"});
}

##########################################
# RELOAD CONFIGURATION                   #
##########################################

if ( $DepPl::args{"force"} ne "en" ) {
	DepPl::loadconfig($DepPl::confdir);
} else {
	undef ($DepPl::args{"force"});
}

DepPl::loadconfig ($userconfdir);

##########################################
# REACT ON DO-SOMETHING-THEN-EXIT PARAMS #
##########################################
# act on --version
if ($DepPl::args{"version"}) {
	print $copyright;
	exit;
}

# act on --license
if ($DepPl::args{"license"}) {
	print $banner;
	DepPl::display_license();
	exit;
}

# act on --help
if (@ARGV == 0 || $DepPl::args{"help"} || !$optval ) {
	print $banner;
	print $DepPl::messages{"help"};
	exit;
}

# if not quiet, display banner
if (!$DepPl::args{"quiet"}) {
	print $banner;
}

##########################################
# COLLECT INFORMATION                    #
##########################################

foreach $file (@ARGV) {

	if (!$DepPl::args{"quiet"} && $DepPl::args{"status"}) {
		printf $DepPl::messages{"msg_checking"}, $file;
	}

	($newfn, $result) = depfile($file);

	if (!$DepPl::args{"quiet"} && $DepPl::args{"status"}) {
		print " ".$result."\n";
	}

	if ( $DepPl::args{"status"} && !$DepPl::args{"quiet"} &&
	     $DepPl::args{"report"} && !$DepPl::args{"libs"} &&
	     $result eq $DepPl::messages{"msg_ok"} &&
	     keys (%{$deps{$newfn}}) > 0 ) {
		printf $DepPl::messages{"msg_depends"}."\n", $file;
		foreach $path ( keys %{$deps{$newfn}}) {
			print "\t$path\n";
		}
	}
}

$stats{"end_time"}=time;

if (keys (%libs) > 0 && $DepPl::args{"status"}) {
	if (!$DepPl::args{"quiet"} && !$DepPl::args{"err"}) {
		printf "\n".$DepPl::messages{"msg_result"}."\n";
	}

	foreach $lib (keys %libs) {
		if (!$DepPl::args{"quiet"} && !$DepPl::args{"err"}) {
			print STDOUT "\t";
		}
		print STDOUT $lib;
		if ($DepPl::args{"bare"}) {
			print STDOUT " ";
		} else {
			print STDOUT "\n";
		}
	}
}

if ( (keys (%misslibs) > 0) && !$DepPl::args{"quiet"} &&
      !$DepPl::args{"err"} && $DepPl::args{"status"}) {
	printf "\n".$DepPl::messages{"msg_misslibs"}."\n";

	foreach $lib (keys %misslibs) {
		print STDOUT "\t".$lib."\n";
	}
}

if (!$DepPl::args{"quiet"} && $DepPl::args{"status"}) {
	print STDOUT "\n";
}

if ( defined ($DepPl::args{"stat"}) ) {
	$stats{"needed_libraries"}=keys (%libs);
	$stats{"missing_libraries"}=keys (%misslibs);
	$stats{"files_checked"}=keys (%deps);
	if ( $DepPl::args{"stat"} >= 0 ) {
		my $tmp;
		printf $DepPl::messages{"stat_found"}."\n", $stats{"file_found"};
		printf $DepPl::messages{"stat_checked"}."\n", $stats{"files_checked"};
		printf $DepPl::messages{"stat_needlibs"}."\n", $stats{"needed_libraries"};
		printf $DepPl::messages{"stat_misslibs"}."\n", $stats{"missing_libraries"};
		if (defined ($tmp=DepPl::format_time ($stats{"end_time"}-$stats{"start_time"})) ) {
			printf $DepPl::messages{"stat_runtime"}."\n", $tmp;
		}
		print "\n";
	}
	if ( $DepPl::args{"stat"} >= 1 ) {
		my @sorted=sort {$libs{$b}<=>$libs{$a}} keys %libs;
		my $tmp=$#sorted;
		if ($tmp>9) {
			$tmp=9;
		}
		if ($tmp>0) {
			print DepPl::pad(sprintf ($DepPl::messages{"stat_top"}, $tmp+1), 
			      int ( 25+length( (sprintf $DepPl::messages{"stat_top"}, $tmp+1) )/2), " ")."\n";
			print "-"x50;
			print "\n";
			my $tmp2;
			for ($tmp2=0; $tmp2<=$tmp; $tmp2++) {
				printf "%s. %s %s\n", DepPl::pad ($tmp2+1, 2, " "),
						      DepPl::pad($sorted[$tmp2], 40, " ", "right"),
						      DepPl::pad ($libs{$sorted[$tmp2]}, 4, " ");
			}
			print "\n";
		}
	}
}

if (!$DepPl::args{"quiet"}) {
	print STDOUT $DepPl::messages{"msg_bye"};
}

exit 0;

##########################################
# SUBROUTINES                            #
##########################################
# depfile ( [file] )
sub depfile {
	my $line;
	my @res;
	my $libname;
	my $pathndummy;
	my $path;
	my $fn=shift;
	my $dummy;

	$stats{"file_found"}++;

	if ($fn !~ /^\//) {
		$fn=$cwd."/".$fn;
	}

	$fn=DepPl::resolve_path($fn);

	if ( ! -e $fn ) {
		return ($fn, $DepPl::messages{"err_missing"});
	}

	my $fileres;

	if ( -d $fn ) {
		$fileres="dir";
	}

	if ( ! -f $fn && ! -d $fn && ! -l $fn ) {
		return ($fn, $DepPl::messages{"err_invtype"});
	}

	if (defined ($dummy=$DepPl::args{"chroot"}) ) {
		if ( $fn!~/^$dummy/) {
			$fn=DepPl::resolve_path($DepPl::args{"chroot"}.$fn);
		}
	}

	if (defined ($dummy=$DepPl::args{"check-area"}) ) {
		if ( $fn!~/^$dummy/) {
			return ($fn, $DepPl::messages{"err_outside"});
		}
	}

	if (defined($DepPl::args{"exclude"})) {
		foreach $dummy (keys %exclude) {
			if ( ($fn=~/$dummy/ && $dummy!~/^\//) || ($fn=~/^$dummy/ && $dummy=~/^\//) ) {
				return ($fn, $DepPl::messages{"msg_skip"});
			}
		}
	}

	if (defined ($fileres) && $fileres eq "dir") {
		if ($DepPl::args{"recursive"}) {
			my @newfiles=DepPl::dir_contents($fn);
			foreach $newfile (@newfiles) {
				chomp ($newfile);
				$dummy=DepPl::resolve_path($fn."/".$newfile);
				if (defined ($DepPl::args{"chroot"}) &&
				    $dummy!~/^$DepPl::args{"chroot"}/) {
					$dummy=$DepPl::args{"chroot"}.$dummy;
				}
				if (!defined ($checked{$dummy}) || 
				    (defined($checked{$dummy}) && $checked{$dummy}<1) ) {
					$checked{$dummy}++;
					push (@ARGV, $dummy);
				}
			}
		}
		return ($fn, $DepPl::messages{"msg_directory"});
	}

	if ( defined ( $checked{$fn} ) && $checked{$fn}>1 ) {
		return ($fn, $DepPl::messages{"msg_ok"});
	}

	$checked{$fn}++;

	@res=split(/\n/,`$DepPl::config{"cmdline_ldd"} $fn 2>/dev/null`);

	foreach $line (@res) {
		($libname,$pathndummy)=split(/\s=>\s/,$line);
		if ($pathndummy) {
			($path,$dummy)=split(/\s+/,$pathndummy);
			if ($path && $path=~/^\//) {
				${$deps{$fn}{$path}}++;
				$libs{$path}++;
			} else {
				$libname=~s/^\s+//g;
				$libname=~s/\s+$//g;
				$misslibs{$libname}++;
				${$deps{$fn}{$libname}}++;
			}
		}
	}

	return ($fn, $DepPl::messages{"msg_ok"});
}

__END__
